package com.ruoyi.gysl.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.itextpdf.html2pdf.ConverterProperties;
import com.itextpdf.html2pdf.HtmlConverter;
import com.itextpdf.io.image.ImageData;
import com.itextpdf.io.image.ImageDataFactory;
import com.itextpdf.kernel.colors.ColorConstants;
import com.itextpdf.kernel.geom.PageSize;
import com.itextpdf.kernel.pdf.PdfDocument;
import com.itextpdf.kernel.pdf.PdfReader;
import com.itextpdf.kernel.pdf.PdfWriter;
import com.itextpdf.kernel.utils.PdfMerger;
import com.itextpdf.layout.Document;
import com.itextpdf.layout.element.AreaBreak;
import com.itextpdf.layout.element.Image;
import com.itextpdf.layout.element.Paragraph;
import com.itextpdf.layout.properties.HorizontalAlignment;
import com.itextpdf.layout.properties.TextAlignment;
import com.ruoyi.common.config.RuoYiConfig;
import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.file.FileUtils;
import com.ruoyi.gysl.entity.GyslProjectHandbook;
import com.ruoyi.gysl.entity.response.BasicInformationResponse;
import com.ruoyi.gysl.mapper.BasicInformationMapper;
import com.ruoyi.gysl.mapper.ProjectHandBookMapper;
import com.ruoyi.gysl.service.GyslProjectHandbookService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.net.MalformedURLException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.rmi.ServerError;
import java.util.Arrays;
import java.util.List;

/**
 * 项目手册(GyslProjectHandbook)表服务实现类
 *
 * @author makejava
 * @since 2025-04-14 13:56:59
 */
@Service("gyslProjectHandbookService")
public class GyslProjectHandbookServiceImpl extends ServiceImpl<ProjectHandBookMapper, GyslProjectHandbook> implements GyslProjectHandbookService {

    @Resource
    private BasicInformationMapper basicInformationMapper;


    @Override
    public void generatePdfs(HttpServletResponse response, Integer id) throws IOException {
        // 获取数据
        GyslProjectHandbook handbook = getById(id);
        List<BasicInformationResponse> basicInfoList = basicInformationMapper.idListToProject(
                Arrays.asList(handbook.getXmId().split(",")));

        // 设置响应头
        response.setContentType("application/pdf");
        response.setHeader("Content-Disposition", "attachment; filename=\"product_catalog.pdf\"");

        try (OutputStream outputStream = response.getOutputStream();
             PdfWriter writer = new PdfWriter(outputStream);
             PdfDocument mergedPdf = new PdfDocument(writer)) { // 主文档生命周期由外部try控制

            // ========== 封面页 ==========
            Document coverDoc = new Document(mergedPdf);
            addCoverPage(coverDoc, handbook);
            coverDoc.close(); // 关闭封面Document但不关闭PdfDocument

            // ========== 动态内容合并 ==========
            String htmlContent = "<!DOCTYPE html><html><body style='font-family: SimSun;'>动态内容</body></html>";
            try (ByteArrayOutputStream tempStream = new ByteArrayOutputStream()) {
                // HTML转PDF
                HtmlConverter.convertToPdf(htmlContent, tempStream);
                // 合并到主文档
                try (PdfReader tempReader = new PdfReader(new ByteArrayInputStream(tempStream.toByteArray()));
                     PdfDocument tempPdf = new PdfDocument(tempReader)) {
                    PdfMerger merger = new PdfMerger(mergedPdf);
                    merger.merge(tempPdf, 1, tempPdf.getNumberOfPages());
                }
            }

        } catch (Exception e) {
            response.reset();
            response.setContentType("application/json");
            response.setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
            throw new ServiceException("错误!");
        }
    }

    @Override
    public void addCoverPage(Document document, GyslProjectHandbook handbook) throws MalformedURLException {
        // 标题
        Paragraph title = new Paragraph(handbook.getName())
                .setFontSize(30).setBold().setFontColor(ColorConstants.BLUE)
                .setTextAlignment(TextAlignment.CENTER)
                .setMarginTop(60);
        document.add(title);

        // 副标题
        Paragraph subtitle = new Paragraph(handbook.getSubtitle())
                .setFontSize(22).setFontColor(ColorConstants.BLUE)
                .setTextAlignment(TextAlignment.RIGHT)
                .setMarginTop(70);
        document.add(subtitle);

        // 封面图片
        String imgPath = RuoYiConfig.getProfile() +
                StringUtils.substringAfter(handbook.getCoverImg(), Constants.RESOURCE_PREFIX);

        if (!FileUtils.checkAllowDownload(imgPath)) {
            throw new ServiceException("非法文件路径: " + imgPath);
        }

        ImageData imageData = ImageDataFactory.create(imgPath);
        Image image = new Image(imageData)
                .setWidth(400)
                .setMarginTop(85)
                .setHorizontalAlignment(HorizontalAlignment.CENTER);
        document.add(image);

        // 分页控制（关键修改：直接操作PdfDocument）
        document.getPdfDocument().addNewPage();
    }
}

